==> yoomoney-1.0.1.tar.gz: yoomoney/src/PluginForm/YooKassa/PaymentOffsiteForm.php <==
<?php

namespace Drupal\yookassa\PluginForm\YooKassa;

use Drupal;
use Drupal\commerce\Response\NeedsRedirectException;
use Drupal\commerce_order\Adjustment;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce_order\Plugin\Field\FieldType\AdjustmentItemList;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\yookassa\Oauth\YooKassaClientFactory;
use Drupal\yookassa\Plugin\Commerce\PaymentGateway\YooKassa;
use Exception;
use YooKassa\Common\Exceptions\ApiException;
use YooKassa\Model\Payment\ConfirmationType;
use YooKassa\Model\Payment\Payment;
use YooKassa\Request\Payments\CreatePaymentRequest;

/**
 * Класс методов для работы с формой оплаты
 */
class PaymentOffsiteForm extends BasePaymentOffsiteForm
{
    /**
     * Конструктор формы для оплаты.
     *
     * @param array $form Массив, содержащий исходную структуру формы плагина
     * @param FormStateInterface $form_state Текущее состояние формы
     *
     * @return array
     * @throws InvalidPluginDefinitionException
     * @throws EntityStorageException
     * @throws NeedsRedirectException
     * @throws Exception
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state): array
    {
        try {
            $form = parent::buildConfigurationForm($form, $form_state);

            /** @var PaymentInterface $payment */
            $payment = $this->entity;
            /** @var YooKassa $paymentGatewayPlugin */
            $paymentGatewayPlugin = $payment->getPaymentGateway()->getPlugin();
            $order                = $payment->getOrder();
            $amount               = $order->getTotalPrice();
            $config               = $paymentGatewayPlugin->getConfiguration();
            $client               = YooKassaClientFactory::getYooKassaClient($config);

            $builder = CreatePaymentRequest::builder()
                ->setAmount($amount->getNumber())
                ->setCapture(true)
                ->setDescription($this->createDescription($order, $config))
                ->setConfirmation(array(
                    'type'      => ConfirmationType::REDIRECT,
                    'returnUrl' => $form['#return_url'],
                ))
                ->setMetadata(array(
                    'cms_name'       => 'yoo_api_drupal10',
                    'module_version' => YooKassa::YOOMONEY_MODULE_VERSION,
                ));

            if ($config['receipt_enabled'] == 1) {
                $profileEmail = $order->get('mail')->getString();
                $builder->setReceiptEmail($profileEmail);
                $items = $order->getItems();
                /** @var OrderItem $item */
                foreach ($items as $item) {
                    /** @var AdjustmentItemList $adjustments */
                    $adjustments = $item->get('adjustments');

                    $taxUuid    = null;
                    $percentage = 0;
                    foreach ($adjustments->getValue() as $adjustmentValue) {
                        /** @var Adjustment $adjustment */
                        $adjustment = $adjustmentValue['value'];
                        if ($adjustment->getType() == 'tax') {
                            $sourceId   = explode('|', $adjustment->getSourceId());
                            $taxUuid    = $sourceId[2];
                            $percentage = $adjustment->getPercentage();
                        }
                    }
                    if (in_array($taxUuid, array_keys($config['yookassa_tax']))) {
                        $vat_code = $config['yookassa_tax'][$taxUuid];
                    } else {
                        $vat_code = $config['default_tax'];
                    }

                    $priceWithTax = $item->getUnitPrice()->getNumber() * (1 + $percentage);
                    $builder->addReceiptItem($item->getTitle(), $priceWithTax, $item->getQuantity(), $vat_code, $config['default_payment_mode'], $config['default_payment_subject']);
                }
            }
            $paymentRequest = $builder->build();
            if (($config['receipt_enabled'] == 1) && $paymentRequest->getReceipt() !== null) {
                $paymentRequest->getReceipt()->normalize($paymentRequest->getAmount());
            }
            $response = $client->createPayment($paymentRequest);

            $payment_storage = Drupal::entityTypeManager()->getStorage('commerce_payment');
            $payments        = $payment_storage->loadByProperties(['order_id' => $order->id()]);
            if ($payments) {
                $payment = reset($payments);
                $payment->enforceIsNew(false);
            }
            $payment->setRemoteId($response->getId());
            $payment->setRemoteState($response->getStatus());
            $payment->save();
            $redirect_url = $response->confirmation->confirmationUrl;
            $data         = [
                'return' => $form['#return_url'],
                'cancel' => $form['#cancel_url'],
                'total'  => $payment->getAmount()->getNumber(),
            ];

            return $this->buildRedirectForm($form, $form_state, $redirect_url, $data);
        } catch (ApiException $e) {
            Drupal::logger('yookassa')->error('Error create payment. Api error: ' . $e->getMessage());
            throw new PaymentGatewayException();
        }
    }

    /**
     * Формирует описание к платежу, подставляет данные в шаблон и обрезает по заданную длину.
     *
     * @param OrderInterface $order Модель заказа
     * @param array $config Конфигурация платежного шлюза
     *
     * @return string
     */
    private function createDescription(OrderInterface $order, array $config): string
    {
        $descriptionTemplate = !empty($config['description_template'])
            ? $config['description_template']
            : t('Оплата заказа №%order_id%');

        $replace = [];
        foreach ($order as $property => $fieldItems) {
            foreach ($fieldItems as $fieldItem) {
                if (!($fieldItem instanceof FieldItemInterface)) {
                    continue;
                }
                $params = $fieldItem->getEntity()->toArray();
                if (empty($params[$property])) {
                    continue;
                }
                if (!is_array($params[$property])) {
                    continue;
                }
                if (empty($params[$property][0])) {
                    continue;
                }
                $fieldData = $params[$property][0];
                if (!is_array($fieldData)) {
                    continue;
                }
                $value = current($fieldData);
                if (!is_scalar($value)) {
                    continue;
                }
                $replace['%'.$property.'%'] = $value;
            }
        }

        $description = strtr($descriptionTemplate, $replace);

        return mb_substr($description, 0, Payment::MAX_LENGTH_DESCRIPTION);
    }

}

==> yoomoney-1.0.2.tar.gz: yoomoney/src/PluginForm/YooKassa/PaymentOffsiteForm.php <==
<?php

namespace Drupal\yookassa\PluginForm\YooKassa;

use Drupal;
use Drupal\commerce\Response\NeedsRedirectException;
use Drupal\commerce_order\Adjustment;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce_order\Plugin\Field\FieldType\AdjustmentItemList;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\yookassa\Oauth\YooKassaClientFactory;
use Drupal\yookassa\Plugin\Commerce\PaymentGateway\YooKassa;
use Exception;
use YooKassa\Common\Exceptions\ApiException;
use YooKassa\Model\Payment\ConfirmationType;
use YooKassa\Model\Payment\Payment;
use YooKassa\Request\Payments\CreatePaymentRequest;

/**
 * Класс методов для работы с формой оплаты
 */
class PaymentOffsiteForm extends BasePaymentOffsiteForm
{
    /**
     * Конструктор формы для оплаты.
     *
     * @param array $form Массив, содержащий исходную структуру формы плагина
     * @param FormStateInterface $form_state Текущее состояние формы
     *
     * @return array
     * @throws InvalidPluginDefinitionException
     * @throws EntityStorageException
     * @throws NeedsRedirectException
     * @throws Exception
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state): array
    {
        try {
            $form = parent::buildConfigurationForm($form, $form_state);

            /** @var PaymentInterface $payment */
            $payment = $this->entity;
            /** @var YooKassa $paymentGatewayPlugin */
            $paymentGatewayPlugin = $payment->getPaymentGateway()->getPlugin();
            $order                = $payment->getOrder();
            $amount               = $order->getTotalPrice();
            $config               = $paymentGatewayPlugin->getConfiguration();
            $client               = YooKassaClientFactory::getYooKassaClient($config);

            $builder = CreatePaymentRequest::builder()
                ->setAmount($amount->getNumber())
                ->setCapture(true)
                ->setDescription($this->createDescription($order, $config))
                ->setConfirmation(array(
                    'type'      => ConfirmationType::REDIRECT,
                    'returnUrl' => $form['#return_url'],
                ))
                ->setMetadata(array(
                    'cms_name'       => YooKassa::CMS_NAME,
                    'module_version' => YooKassa::YOOMONEY_MODULE_VERSION,
                ))
                ->setTaxSystemCode($config['default_tax_rate']);

            if ($config['receipt_enabled'] == 1) {
                $profileEmail = $order->get('mail')->getString();
                $builder->setReceiptEmail($profileEmail);
                $items = $order->getItems();
                /** @var OrderItem $item */
                foreach ($items as $item) {
                    /** @var AdjustmentItemList $adjustments */
                    $adjustments = $item->get('adjustments');

                    $taxUuid    = null;
                    $percentage = 0;
                    foreach ($adjustments->getValue() as $adjustmentValue) {
                        /** @var Adjustment $adjustment */
                        $adjustment = $adjustmentValue['value'];
                        if ($adjustment->getType() == 'tax') {
                            $sourceId   = explode('|', $adjustment->getSourceId());
                            $taxUuid    = $sourceId[2];
                            $percentage = $adjustment->getPercentage();
                        }
                    }
                    if (in_array($taxUuid, array_keys($config['yookassa_tax']))) {
                        $vat_code = $config['yookassa_tax'][$taxUuid];
                    } else {
                        $vat_code = $config['default_tax'];
                    }

                    $priceWithTax = $item->getUnitPrice()->getNumber() * (1 + $percentage);
                    $builder->addReceiptItem($item->getTitle(), $priceWithTax, $item->getQuantity(), $vat_code, $config['default_payment_mode'], $config['default_payment_subject']);
                }
            }
            $paymentRequest = $builder->build();
            if (($config['receipt_enabled'] == 1) && $paymentRequest->getReceipt() !== null) {
                $paymentRequest->getReceipt()->normalize($paymentRequest->getAmount());
            }
            $response = $client->createPayment($paymentRequest);

            $payment_storage = Drupal::entityTypeManager()->getStorage('commerce_payment');
            $payments        = $payment_storage->loadByProperties(['order_id' => $order->id()]);
            if ($payments) {
                $payment = reset($payments);
                $payment->enforceIsNew(false);
            }
            $payment->setRemoteId($response->getId());
            $payment->setRemoteState($response->getStatus());
            $payment->save();
            $redirect_url = $response->confirmation->confirmationUrl;
            $data         = [
                'return' => $form['#return_url'],
                'cancel' => $form['#cancel_url'],
                'total'  => $payment->getAmount()->getNumber(),
            ];

            return $this->buildRedirectForm($form, $form_state, $redirect_url, $data);
        } catch (ApiException $e) {
            Drupal::logger('yookassa')->error('Error create payment. Api error: ' . $e->getMessage());
            throw new PaymentGatewayException();
        }
    }

    /**
     * Формирует описание к платежу, подставляет данные в шаблон и обрезает по заданную длину.
     *
     * @param OrderInterface $order Модель заказа
     * @param array $config Конфигурация платежного шлюза
     *
     * @return string
     */
    private function createDescription(OrderInterface $order, array $config): string
    {
        $descriptionTemplate = !empty($config['description_template'])
            ? $config['description_template']
            : t('Оплата заказа №%order_id%');

        $replace = [];
        foreach ($order as $property => $fieldItems) {
            foreach ($fieldItems as $fieldItem) {
                if (!($fieldItem instanceof FieldItemInterface)) {
                    continue;
                }
                $params = $fieldItem->getEntity()->toArray();
                if (empty($params[$property])) {
                    continue;
                }
                if (!is_array($params[$property])) {
                    continue;
                }
                if (empty($params[$property][0])) {
                    continue;
                }
                $fieldData = $params[$property][0];
                if (!is_array($fieldData)) {
                    continue;
                }
                $value = current($fieldData);
                if (!is_scalar($value)) {
                    continue;
                }
                $replace['%'.$property.'%'] = $value;
            }
        }

        $description = strtr($descriptionTemplate, $replace);

        return mb_substr($description, 0, Payment::MAX_LENGTH_DESCRIPTION);
    }

}
